@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-from project import current_project, Model
+from project import current_project
 
 __all__ = ['util']
 
